# -*- encoding: utf-8 -*-

import unittest
import os

import productmd.compose
from compose_utils.changelog import ComposeChangelog


def _get_compose_path(ident):
    base = os.path.dirname(__file__)
    pth = os.path.join(base, 'composes', ident)
    return pth


DUMMY_FIREFOX_SIZE = 5966 + 5934
OLD_DUMMY_FIREFOX_SIZE = 5878 + 5842
DUMMY_ELINKS_SIZE = 5850 + 5826 + 5818
DUMMY_TFTP_SIZE = 5842 + 5814 + 5806

DUMMY_FIREFOX = {
    'size': DUMMY_FIREFOX_SIZE,
    'name': 'Dummy-firefox',
    'old_nvr': 'Dummy-firefox-16.0.1-1',
    'old_rpms': ['Dummy-firefox'],
    'common_rpms': ['Dummy-firefox'],
    'rpms': ['Dummy-firefox'],
    'changelog': ['* Tue Mar 15 2016 Lubomír Sedlář <lubomir.sedlar@gmail.com> - 1:0.1.0-1\n- new version'],
    'added_rpms': [],
    'dropped_rpms': [],
    'nvr': 'Dummy-firefox-1:0.1.0-1',
    'summary': 'A dummy firefox package',
    'size_change': DUMMY_FIREFOX_SIZE - OLD_DUMMY_FIREFOX_SIZE,
}

DUMMY_ELINKS = {
    'size': DUMMY_ELINKS_SIZE,
    'name': 'dummy-elinks',
    'rpms': ['dummy-elinks'],
    'nvr': 'dummy-elinks-2.6-2',
    'summary': 'A dummy elinks package',
}

DUMMY_TFTP = {
    'size': DUMMY_TFTP_SIZE,
    'name': 'dummy-tftp',
    'rpms': ['dummy-tftp'],
    'nvr': 'dummy-tftp-5.2-6',
    'summary': 'A dummy tftp package',
}


class ChangelogTest(unittest.TestCase):

    def test_changelog(self):
        old_compose = productmd.compose.Compose(_get_compose_path('DP-1.0-20160315.t.0'))
        new_compose = productmd.compose.Compose(_get_compose_path('DP-1.0-20160315.t.1'))
        changelog = ComposeChangelog()

        data = changelog.get_changelog(old_compose, new_compose)
        self.assertEqual(data['old_compose'], 'DP-1.0-20160315.t.0')
        self.assertEqual(data['new_compose'], 'DP-1.0-20160315.t.1')
        self.assertItemsEqual(data['added_packages'], [DUMMY_ELINKS])
        self.assertItemsEqual(data['dropped_packages'], [DUMMY_TFTP])
        self.assertItemsEqual(data['upgraded_packages'], [DUMMY_FIREFOX])
        self.assertItemsEqual(data['downgraded_packages'], [])
        self.assertDictEqual(data['added_images'][0], {
            "arch": "s390x",
            "bootable": False,
            "checksums": {
                "md5": "2d0a0e02d6a2225fba2c4cdf41d45ff7",
                "sha1": "8788b0aa4974440f7ca12c5dd0cc6dfc78314fdd",
                "sha256": "1b756cddb139e57fd6a07af72a7d52a9dc01c5fb0b9d37ef16fdf73a1a0a19af"
            },
            "disc_count": 1,
            "disc_number": 1,
            "format": "iso",
            "implant_md5": "0a2fc4f0e6f926c860e5df6c5ff0df32",
            "mtime": 1458031712,
            "path": "Server/s390x/iso/DP-1.0-20160315.t.1-Server-s390x-dvd1.iso",
            "size": 493568,
            "type": "dvd",
            "volume_id": "DP-1.0 Server.s390x",
            "subvariant": "Server"
        })
        self.assertDictEqual(data['dropped_images'][0], {
            "arch": "x86_64",
            "bootable": False,
            "checksums": {
                "md5": "7ea0d315e4b51006eb7c1de3da763c55",
                "sha1": "268bf671d5e62f35b70a2db5b1198b861d4b0eb8",
                "sha256": "32fedd333df5a0c78f3dd8c2634f8a4ab2328c7e0c9904dd06ac865fa723134f"
            },
            "disc_count": 1,
            "disc_number": 1,
            "format": "iso",
            "implant_md5": "128abd11392887a8abb2bbf119889791",
            "mtime": 1458031335,
            "path": "Client/x86_64/iso/DP-1.0-20160315.t.0-Client-x86_64-dvd1.iso",
            "size": 516096,
            "type": "dvd",
            "volume_id": "DP-1.0 Client.x86_64",
            "subvariant": "Client"
        })
        self.assertDictEqual(data['summary'], {
            'added_images': 1,
            'dropped_images': 1,
            'added_packages': 1,
            'added_packages_size': DUMMY_ELINKS_SIZE,
            'upgraded_packages': 1,
            'upgraded_packages_size': DUMMY_FIREFOX_SIZE,
            'upgraded_packages_size_change': DUMMY_FIREFOX_SIZE - OLD_DUMMY_FIREFOX_SIZE,
            'dropped_packages': 1,
            'dropped_packages_size': DUMMY_TFTP_SIZE,
            'downgraded_packages': 0,
            'downgraded_packages_size': 0,
            'downgraded_packages_size_change': 0,
        })

if __name__ == '__main__':
    unittest.main()
